#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/ldapclient-update.sh - ldapclient parameter update script
#
# Copyright (c) 2010-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2010-01-17 jed
# Last Update:  $Id: ldapclient-update.sh 72519 2020-02-02 14:00:31Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/jedlib
. /var/install/include/configlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --info --force-output "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for new parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
    local _vs_var1
    local _vs_var2
    local _vs_ret

    eval _vs_var1=\$"${1}"
    eval _vs_var2=\$"{${1}+EMPTY}"

    if [ -z "${_vs_var1}" ]
    then
        if [ "${_vs_var2}" = 'EMPTY' ]
        then
            # variable is empty, but set
            _vs_ret=0
        else
            # variable is not set
            _vs_ret=1
        fi
    else
        # variable has a value
        _vs_ret=0
    fi

    return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "ldapclient" "2010-01-17" "jed" "Juergen Edner"
        #------------------------------------------------------------------------------

        if [ "${LDAPCLIENT_DO_DEBUG}" = "yes" -o "${LDAPCLIENT_DO_DEBUG}" = "no" ]
        then
            # debug active
            printvar "LDAPCLIENT_DO_DEBUG" "debug mode: yes or no"
            echo
        else
            # debug parameter exists but not active
            if [ -n "`grep LDAPCLIENT_DO_DEBUG ${source_conf}`" ]
            then
                echo "# LDAPCLIENT_DO_DEBUG='yes'                 # debug mode: yes or no"
                echo
            fi
        fi

        echo
        #------------------------------------------------------------------------------
        printgroup "General settings"
        #------------------------------------------------------------------------------
        printvar "START_LDAPCLIENT"                "activate configuration: yes or no"
        echo
        printvar "LDAPCLIENT_LDAP_HOSTURI"         "host-uri of ldap server"
        printvar "LDAPCLIENT_LDAP_BASEDC"          "base domain component"
        printvar "LDAPCLIENT_LDAP_BASEDN"          "ldap search base"
        printvar "LDAPCLIENT_LDAP_AUTH"            "use ldap authentication: yes or no"
        printvar "LDAPCLIENT_LDAP_ADMIN_NAME"      "name of LDAP administrator"
        printvar "LDAPCLIENT_LDAP_ADMIN_PASS"      "password of LDAP administrator"

        #------------------------------------------------------------------------------
        printgroup "Security settings"
        #------------------------------------------------------------------------------
        printsetvar "LDAPCLIENT_CLIENT_CERT_FILE"  "name of client certificate"
        printsetvar "LDAPCLIENT_CLIENT_KEY_FILE"   "name of client certificate key"
        printsetvar "LDAPCLIENT_SERVER_CERT_CHECK" "check certificate: never, allow, try, hard"

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}

    # set access rights
    chmod 0644 ${generate_conf}
    chown root ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

basefile=/etc/config.d/base
configfile=/etc/config.d/ldapclient
installfile=/var/run/ldapclient.install
tmpfile=/var/run/ldapclient.$$

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case "$1" in
    update )
        goflag=1
        ;;
    test )
        source_conf=${configfile}

        generate_conf=/etc/config.d/ldapclient.test
        goflag=1
        ;;
    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  ldapclient-update.sh [update] - the file ${configfile}.import will be read, the configuration will"
        echo "                                  be checked and an updated mail configuration file will be written."
        echo
        goflag=0
        ;;
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists
        myecho --pack ${module_name} --info "previous configuration found ..."
        . ${basefile}
        . ${source_conf}

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
    fi

    if [ ${modflag} -eq 1 ]
    then
        myanykey --pack ${module_name}
    fi
fi

#==============================================================================
# end
#==============================================================================
exit 0
