#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/ldapclient.sh - configuration generator script for OpenLDAP
#
# Copyright (c) 2009-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2010-01-17 jv
# Last Update:  $Id: ldapclient.sh 72544 2020-02-02 18:04:36Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

#include eislib etc.
. /var/install/include/check-eisfair-version
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# check directory access
#------------------------------------------------------------------------------
check_directories ()
{
    # create config path
    if [ ! -d ${ldap_confdir} ]
    then
        mkdir -p ${ldap_confdir}
    fi

    chown -R ${user}  ${ldap_confdir}
    chgrp -R ${group} ${ldap_confdir}
    chmod    0555     ${ldap_confdir}
}

#------------------------------------------------------------------------------
# create LDAP client configuration - ldap.conf
#------------------------------------------------------------------------------
create_ldap_conf ()
{
    myecho --pack ${module_name} "Creating LDAP client configuration ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_ldapconf_file}" "${pgmname}" "${module_name}" "${ldapclient_version}"
        #-------------------------------------------------------------------------
      # echo 'ldap_version 3'
      # echo 'pam_password crypt'

        echo "URI     ${LDAPCLIENT_LDAP_HOSTURI}"
        echo "BASE    ${LDAPCLIENT_LDAP_BASEDN}"

        # TLS_CACERTDIR <path> - Specifies the path of a directory that contains
        #                        Certificate Authority certificates in separate
        #                        individual files. The TLS_CACERT is always used
        #                        before TLS_CACERTDIR.
        echo "TLS_CACERTDIR ${sslcert_path}"

        # TLS_REQCERT <level> - Specifies what checks to perform on server
        #                       certificates in a TLS session, if any. The <level>
        #                       can be specified as one of the following keywords:
        #
        # never - The client will not request or check any server certificate.
        # allow - The server certificate is requested. If no certificate is
        #         provided, the session proceeds normally. If a bad certificatei
        #         is provided, it will be ignored and the session proceeds normally.
        # try   - The server certificate is requested. If no certificate is provided,
        #         the session proceeds normally. If a bad certificate is provided,
        #         the session is immediately terminated.
        # demand|hard - These keywords are equivalent. The server certificate is
        #               requested. If no certificate is provided, or a bad
        #               certificate is provided, the session is immediately
        #               terminated. This is the default setting.
        if [ -n "${LDAPCLIENT_SERVER_CERT_CHECK}" ]
        then
            echo "TLS_REQCERT   ${LDAPCLIENT_SERVER_CERT_CHECK}"
        fi

    } > ${generate_ldapconf_file}

    chown ${user}  ${generate_ldapconf_file}
    chgrp ${group} ${generate_ldapconf_file}
    chmod 0644     ${generate_ldapconf_file}
}

#------------------------------------------------------------------------------
# create LDAP client private configuration - .ldaprc
#------------------------------------------------------------------------------
create_ldaprc_conf ()
{
    if [ -n "${LDAPCLIENT_CLIENT_CERT_FILE}" ]
    then
        myecho --pack ${module_name} "Creating LDAP private client configuration ..."

        {
            #-------------------------------------------------------------------------
            print_short_header "${generate_ldaprcconf_file}" "${pgmname}" "${module_name}" "${ldapclient_version}"
            #-------------------------------------------------------------------------

            # User specific LDAP settings

            # Override global directive (if set)
            #TLS_REQCERT demand

            # client authentication
            if [ -n "${LDAPCLIENT_CLIENT_CERT_FILE}" ]
            then
                # TLS_CERT <filename> - Specifies the file that contains the client
                #                       certificate. This is a user-only option.
                echo "TLS_CERT      ${LDAPCLIENT_CLIENT_CERT_FILE}"

                # TLS_KEY <filename> - Specifies the file that contains the private key that
                #                      matches the certificate stored in the TLS_CERT file.
                #                      This is also a user-only directive.
                echo "TLS_KEY       ${LDAPCLIENT_CLIENT_KEY_FILE}"
            fi
        } > ${generate_ldaprcconf_file}

        chown ${user}  ${generate_ldaprcconf_file}
        chgrp ${group} ${generate_ldaprcconf_file}
        chmod 0600     ${generate_ldaprcconf_file}
    else
        # remove file
        rm -f ${generate_ldaprcconf_file}
    fi
}

#------------------------------------------------------------------------------
# create SASL configuration - sasl3/sasl.conf
#------------------------------------------------------------------------------
create_sasl_conf ()
{
    # create SASL config
    myecho --pack ${module_name} "Creating SASL configuration ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_sasl_file}" "${pgmname}" "${module_name}" "${ldapclient_version}"
        #-------------------------------------------------------------------------

        # Source: http://www.sendmail.org/~ca/email/cyrus2/options.html

        # log_level - Controls the level of verbosity of messages sent to
        #             the syslogd service.
        #
        # 0 - no messages
        # 1 - unusual errors
        # 2 - all authentication errors
        # 3 - log non-fatal warnings
        # 4 - more verbose than 3
        # 5 - more verbose than 4
        # 6 - traces of internal protocols
        # 7 - traces of internal protocols,including passwords
      # echo 'loglevel: 0'

        # pwcheck_method - Specifies one or more password verifcation services
        #                  and/or auxprop-plugins to process authentication.
        #                  Valid values are the names of the password verification
        #                  services or auxprop-plugins.
        echo 'pwcheck_method: auxprop'

        # auxprop_plugin - Name of auxiliary plugin to use, you may specify a
        #                  space-separated list of plugin names, and the plugins
        #                  will be queried in order.
        echo 'auxprop_plugin: ldapdb'

        # mech_list - Specifies a list of mechanisms a Cyrus SASL may offer a
        #             server. Valid values are the names of mechanisms, separated
        #             by whitespace.
        echo 'mech_list: login plain digest-md5 cram-md5'

        # ldapdb_uri - LDAP server uri, you can specify a space-separated list of
        #              URIs - ldaps://ldap1/ ldaps://ldap2/
        echo "ldapdb_uri: ${LDAPCLIENT_LDAP_HOSTURI}"

        if [ "${LDAPCLIENT_LDAP_AUTH}" = 'yes' ]
        then
            # ldapdb_mech - LDAP SASL mechanism for authentication.
            echo 'ldapdb_mech: digest-md5'

            # ldapdb_id - LDAP SASL authentication id.
            echo "ldapdb_id: ${LDAPCLIENT_LDAP_ADMIN_NAME}"

            # ldapdb_pw - LDAP password for SASL authentication id.
            echo "ldapdb_pw: ${LDAPCLIENT_LDAP_ADMIN_PASS}"
        fi

    } > ${generate_sasl_file}

    chown ${user}  ${generate_sasl_file}
    chgrp ${group} ${generate_sasl_file}
    chmod 0600     ${generate_sasl_file}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

### set path names ###
ldap_confdir=/etc/openldap
sslcert_path=/usr/local/ssl/certs

if [ "${EISFAIR_SYSTEM}" = 'eisfair-64' ]
then
    ldap_sasldir=/usr/lib64/sasl3
else
    ldap_sasldir=/usr/lib/sasl3
fi

### set file names ###
basefile=/etc/config.d/base
configfile=/etc/config.d/${module_name}
generate_ldapconf_file=${ldap_confdir}/ldap.conf
generate_ldaprcconf_file=${HOME}/.ldaprc
generate_sasl_file=${ldap_sasldir}/slapd.conf

user='ldap'
group='ldap'

### other parameters ###
if [ "`/usr/bin/eisman check ${module_name}`" = 'installed' ]
then
    ldapclient_version="`/usr/bin/eisman query --installed --fields=version --no-tty ${module_name} | sed -e '/^version/!d' -e 's/^version *//' | tail -n1`"
else
    ldapclient_version='unknown'
fi

. ${configfile}

myecho --pack ${module_name} "${module_name} version: ${ldapclient_version}"

if [ "${START_LDAPCLIENT}" = 'yes' ]
then
    check_directories

    # generate all configuration files

    create_ldap_conf
    create_ldaprc_conf
    create_sasl_conf

    myecho --pack ${module_name} "finished."
else
    myecho --pack ${module_name} --force-output "${module_name} configuration is currently disabled!"
    rm -f ${generate_ldapconf_file}
    rm -f ${generate_sasl_file}
fi

#==============================================================================
# end
#==============================================================================
exit 0
